package com.andujar.springmvc.service;


import java.util.Date;
import java.util.Iterator;
import java.util.List;




import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.andujar.springmvc.domain.AreaActivitat;
import com.andujar.springmvc.domain.AreaAssignatura;
import com.andujar.springmvc.domain.Nivell;
import com.andujar.springmvc.exception.Excepcio;
import com.andujar.springmvc.repository.*;

@Component
@Transactional
public class AreaAssignaturaServiceImpl implements AreaAssignaturaService  {
	
	AreaAssignatura areaAssignaturaClass;

	@Autowired
	private AreaAssignaturaDAO dao;
	
	@Autowired
	private AreaActivitatService areaActivitatService;
	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#guardaAreaAssignatura(com.andujar.springmvc.domain.AreaAssignatura)
	 */
	@Override
	public long guardaAreaAssignatura(AreaAssignatura areaAssignatura){
		return dao.guardaAreaAssignatura  (areaAssignatura);
	}


	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#guardaAreaAssignatura(com.andujar.springmvc.domain.Nivell, java.lang.String, java.lang.String, java.util.Date, java.util.Date)
	 */
	@Override
	public long guardaAreaAssignatura(Nivell nivell, String nom, String descripcio,
			Date data_alta, Date data_baixa) throws Excepcio {
		
		
			
				areaAssignaturaClass = new AreaAssignatura(nivell, nom, descripcio,
						data_alta,data_baixa);
		
		
		return dao.guardaAreaAssignatura  (areaAssignaturaClass);
	}

	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#actualizaAreaAssignatura(com.andujar.springmvc.domain.AreaAssignatura)
	 */
	@Override
	public void actualizaAreaAssignatura(AreaAssignatura areaAssignatura){
		dao.actualizaAreaAssignatura(areaAssignatura);
	}
	

	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#eliminaAreaAssignatura(com.andujar.springmvc.domain.AreaAssignatura)
	 */
	@Override
	public void eliminaAreaAssignatura(AreaAssignatura areaAssignatura){
		dao.eliminaAreaAssignatura(areaAssignatura);
	}
	
	
	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#obteAreaAssignatura(long)
	 */
	@Override
	public AreaAssignatura obteAreaAssignatura(long idAreaAssignatura) {
		return dao.obteAreaAssignatura(idAreaAssignatura);
	}
	

	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.AreaAssignaturaService#obteLlistaAreaAssignatura()
	 */
	@Override
	public List<AreaAssignatura> obteLlistaAreaAssignatura(){
		
		return dao.obteLlistaAreaAssignaturas();
	}
	
	@Override
	public List<AreaAssignatura> obteLlistaAreaAssignaturaAlta(){
		
		return dao.obteLlistaAreaAssignaturasAlta();
	}
	
	@Override
	public List<AreaAssignatura> obteLlistaAreaAssignaturasAltaNivell(long nivellId){
		return dao.obteLlistaAreaAssignaturasAltaNivell(nivellId);
		
	}
	
	@Override
	public void donaBaixaAreaAssignatura (long areaAssignaturaId){
		areaAssignaturaClass = obteAreaAssignatura(areaAssignaturaId);
		areaAssignaturaClass.setDataBaixa(new Date());
		actualizaAreaAssignatura(areaAssignaturaClass);
		
		//Els % de totes les areaActivitats d'una ActivitatAvaluadora han de sumar 100%
		//Si donem de baixa una AreaAssignatura les AreaActivitats que hi esitiguin relacionades deixaran de tenir-se en compte
		//Repartirem el valor del % de cadascuna d'aquestes AreaActivitats entre la resta d'AreaActivitats de la corresponent ActivitatAvaluadora
		//Obt el llistat de AreaActivitats relacionades amb l'AreaAssignatura que acabem de donar de baixa
		
		AreaActivitat areaActivitat;
		AreaActivitat areaActivitatModificar;
		int nombreAreaActivitats;
		double percentatgeRepartir;
		List <AreaActivitat> areasActivitatActivitatAvaluadora;
		
		//Cerquem totes les AreaActivitats que tenien com a AreaAssignatura la que hem donat de baixa
		List <AreaActivitat> areasActivitatAreaAssignatura = areaActivitatService.obteLlistaAreaActivitatsAreaAssignatura(areaAssignaturaId);
		
		
		Iterator <AreaActivitat> areasActivitatAreaAssignaturaIterator = areasActivitatAreaAssignatura.iterator();
		while (areasActivitatAreaAssignaturaIterator.hasNext()){
			areaActivitat = areasActivitatAreaAssignaturaIterator.next();
			
			//Per cada una de les AreaActivitats cerquem totes les que pertanyen a la mateixa ActivitatAvaluador
			areasActivitatActivitatAvaluadora = areaActivitatService.obteLlistaAreaActivitatsActivitatAvaluadora(areaActivitat.getActivitatAvaluadora().getId());
			
			//Repartim el % que tocava a l'AreaActivitat que deixar d'estar activa i el repartim entre les AreaActivitat que pertanyen a la mateixa ActivitatAvaluadora
			nombreAreaActivitats = areasActivitatActivitatAvaluadora.size();
			percentatgeRepartir = areaActivitat.getPercentatgeProgramacio() / (nombreAreaActivitats -1);
			
			Iterator <AreaActivitat> areasActivitatActivitatAvaluadoraIterator = areasActivitatActivitatAvaluadora.iterator();
			while(areasActivitatActivitatAvaluadoraIterator.hasNext()){
				areaActivitatModificar = areasActivitatActivitatAvaluadoraIterator.next();
				if (areaActivitat.getId()!= areaActivitatModificar.getId()){
					areaActivitatModificar.setPercentatgeProgramacio(percentatgeRepartir + areaActivitatModificar.getPercentatgeProgramacio());
					areaActivitatService.actualizaAreaActivitat(areaActivitatModificar);
				}
			}
			
		}
	}
	
	

}
