package com.andujar.springmvc.service;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.andujar.springmvc.domain.ActivitatAvaluadora;
import com.andujar.springmvc.domain.Nivell;
import com.andujar.springmvc.domain.UnitatDidactica;
import com.andujar.springmvc.exception.Excepcio;
import com.andujar.springmvc.repository.*;

@Component
@Transactional
public class UnitatDidacticaServiceImpl implements UnitatDidacticaService  {
	
	UnitatDidactica unitatDidacticaClass;

	@Autowired
	private UnitatDidacticaDAO dao;
	
	@Autowired
	private ActivitatAvaluadoraService activitatAvaluadoraService;
	
	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#guardaUnitatDidactica(com.andujar.springmvc.domain.UnitatDidactica)
	 */
	@Override
	public long guardaUnitatDidactica(UnitatDidactica unitatDidactica){
		return dao.guardaUnitatDidactica  (unitatDidactica);
	}


	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#guardaUnitatDidactica(com.andujar.springmvc.domain.Nivell, java.lang.Long, java.lang.String, java.util.Date, java.util.Date)
	 */
	@Override
	public long guardaUnitatDidactica(Nivell nivell, Long numero, String titol,
			Date data_alta, Date data_baixa) throws Excepcio {
		unitatDidacticaClass = new UnitatDidactica(nivell,numero, titol,
				data_alta,  data_baixa);
		return dao.guardaUnitatDidactica  (unitatDidacticaClass);
	}

	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#actualizaUnitatDidactica(com.andujar.springmvc.domain.UnitatDidactica)
	 */
	@Override
	public void actualizaUnitatDidactica(UnitatDidactica unitatDidactica){
		dao.actualizaUnitatDidactica(unitatDidactica);
	}
	

	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#eliminaUnitatDidactica(com.andujar.springmvc.domain.UnitatDidactica)
	 */
	@Override
	public void eliminaUnitatDidactica(UnitatDidactica unitatDidactica){
		dao.eliminaUnitatDidactica(unitatDidactica);
	}
	
	
	
	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#obteUnitatDidactica(long)
	 */
	@Override
	public UnitatDidactica obteUnitatDidactica(long idUnitatDidactica) {
		return dao.obteUnitatDidactica(idUnitatDidactica);
	}
	

	/* (non-Javadoc)
	 * @see com.andujar.springmvc.service.UnitatDidacticaService#obteLlistaUnitatDidactica()
	 */
	@Override
	public List<UnitatDidactica> obteLlistaUnitatDidactica(){
		
		return dao.obteLlistaUnitatDidacticas();
	}
	
	@Override
	public List<UnitatDidactica> obteLlistaUnitatDidacticaAlta(long nivellId){
		
		return dao.obteLlistaUnitatDidacticasAlta(nivellId);
	}
	
	@Override
	public void donaBaixaUnitatDidactica (long unitatDidacticaId){
		//Marca unitatDidactica amb una data de baixa
		unitatDidacticaClass = obteUnitatDidactica(unitatDidacticaId);
		unitatDidacticaClass.setDataBaixa(new Date());
		actualizaUnitatDidactica(unitatDidacticaClass);
		
		//Dna de baixa totes les activitats avaluadores relacionades amb aquesta unitat didctica
		List <ActivitatAvaluadora> activitats = activitatAvaluadoraService.obteLlistaActivitatAvaluadoraAlta(unitatDidacticaId);
		
		for (ActivitatAvaluadora activitat : activitats){
			activitatAvaluadoraService.donaBaixaActivitatAvaluadora(activitat.getId());
		}		
	}
	
	
	public List<UnitatDidactica> obteLlistaUnitatDidacticasAltaAvaluacio(long nivellId, Date fiAvaluacioAnyAcademic){
		return dao.obteLlistaUnitatDidacticasAltaAvaluacio(nivellId, fiAvaluacioAnyAcademic);
	}

}
